
USE [WaspTrackAsset]

PRINT ''
PRINT ''
PRINT '******************************'
PRINT 'Update tables  -- [schedule] '
PRINT '******************************'
PRINT ''

-- Add the asset_id column to the table
if ((SELECT COUNT(*) FROM dbo.syscolumns WHERE id = OBJECT_ID(N'[dbo].[schedule]') AND NAME = 'asset_id') = 0)
BEGIN
	print 'Adding column [asset_id]'
	ALTER TABLE [dbo].[schedule] ADD [asset_id] int  		 	
END

go

if ((SELECT COUNT(*) FROM dbo.syscolumns WHERE id = OBJECT_ID(N'[dbo].[schedule]') AND NAME = 'maint_description') = 0)
BEGIN
	PRINT 'Altering table schedule' 
	-- Add the description column to the table
	ALTER TABLE [dbo].[schedule] ADD [maint_description] [description_type] 
	-- Move the schedule_id which is the asset_id to the asset_id column
	UPDATE [dbo].[schedule] SET [asset_id] = [schedule_id]
	-- drop the primary key so we can change the column
	ALTER TABLE [dbo].[schedule] DROP CONSTRAINT [PK_schedule]	
	-- modify the schedule_id column to be an identity
	ALTER TABLE [dbo].[schedule] DROP COLUMN [schedule_id] 
	print 'DROP COLUMN [schedule_id]'
	ALTER TABLE [dbo].[schedule] ADD [schedule_id] INT NOT NULL IDENTITY(1,1)
	print 'ALTER COLUMN [schedule_id] IDENTITY'
	
	-- In old versions there might be a note entry without a corresponding maintenance entry
	-- Since we know these are not being used we delete them to make way for the new schedules
	delete from notes where table_name = 'asset_maintenance' and note_id not in (select asset_id from schedule where schedule_type = 'asset_maintenance')
	-- Also delete blank notes
	delete from notes where cast(note_text as nvarchar) = '' and table_name = 'asset_maintenance'

	-- make the asset_id column not null
	ALTER TABLE [dbo].[schedule] ALTER COLUMN [asset_id] int not null	
	
	PRINT 'Altering table schedule complete'
END

go

update schedule set maint_description = 'Asset Maintenance' where maint_description = ''

-- check for the primary key
if ( (SELECT ObjectProperty(object_id('PK_schedule'),'CnstIsDisabled')) is null)
BEGIN
	-- if it is not there add it
	ALTER TABLE [dbo].[schedule] WITH NOCHECK ADD 
	CONSTRAINT [PK_schedule] PRIMARY KEY CLUSTERED 
	(
		[schedule_id]
	)  ON [PRIMARY]
	
	-- put something in the description field
	UPDATE [dbo].[schedule] SET [maint_description] = 'Maintenance Schedule'
END

-- insert the 0 row for all existing assets to make sure maintenance can 
-- be done on an asset even if it does not have a schedule set up
if ((select count(schedule_id) from schedule where occurs_type = 0 and schedule_type = 'asset_maintenance')=0)
BEGIN
	declare @i int
	select @i = min(asset_id) from asset 
	while @i is not null
	begin
		INSERT INTO [dbo].[schedule] ([schedule_type],[occurs_type],[occurs_every],[days_of_week],[day_of_month],[monthly_type],[schedule_date],[assigned_to],[record_status],[company_id],[asset_id],[maint_description])
		VALUES ('asset_maintenance',0,0,'',0,0,'1-1-1900','',1,1,@i,'Unscheduled Maintenance')
		select @i = min(asset_id) from asset where asset_id > @i 
	end
	
	-- Update the notes table so that the note_id = schedule ID not the asset id for all asset_maintenance entries
	update notes set notes.note_id = (Select schedule.schedule_id from schedule where notes.note_id = schedule.asset_id 
	and schedule.maint_description != 'Unscheduled Maintenance' and schedule.schedule_type = 'asset_maintenance' ) where notes.table_name = 'asset_maintenance' 

END

go

-- set all the old maintenance transactions (previous to v6) to 'Unscheduled Maintenance'
-- (or else they will not be in the maintenace history)
declare @t int
declare @t_assetid int
declare @t_userid int
declare @t_companyid int
select @t = min(transaction_id) from asset_transactions where trans_type = 600 and action_type is null
-- loop through all the asset maintenance transactions with action type null (shows they are previous to v6)
while @t is not null
begin

	-- get variables
	select @t_assetid = asset_id, @t_userid = user_id, @t_companyid = company_id from asset_transactions where transaction_id = @t	
	
	-- update the transaction's action type to equal the schedule_id pertaining to the unscheduled maintenance of that asset
	UPDATE [dbo].[asset_transactions] SET action_type = (select schedule_id from schedule where occurs_type = 0 and asset_id = @t_assetid) WHERE transaction_id = @t
	
	-- see if we need to insert the note (on previous versions of MA, if there is no description entered on mobile device, then no note gets inserted)
	if( (SELECT COUNT(note_id) FROM notes WHERE note_id = @t AND table_name = 'asset_transactions') = 0)
	begin
		INSERT INTO [dbo].[notes] (note_id, table_name,note_text,note_date,note_user_id,record_status,company_id) values (@t, 'asset_transactions','',getdate(),@t_userid,1,@t_companyid)
	end
	
	--advance counter
	select @t = min(transaction_id) from asset_transactions where trans_type = 600 and action_type is null and transaction_id > @t
end
go
